;;; ==========================================================================
;;; File     : MACIFY.LSP
;;; Copyright: cad nauseam, 1996.
;;; Author   : Steve Johnson
;;; Purpose  : Prepares an AutoCAD drawing for export to Mac drafting programs.
;;; Version  : 0.5
;;; Date     : 24 February 1996
;;; Requires : AutoCAD Release 11, 12 or 13
;;; ==========================================================================
;;; Function definition structure:
;;; MACIFY.LSP
;;;  C:MACIFY
;;;    local_error
;;;    get_xlay
;;;    add_xref
;;;    move_ps
;;;    report_file
;;;  mex_main
;;;    exblk
;;;      atfix
;;;      atlist
;;;    exdim
;;;    exmtext
;;;    expol
;;;    fixnew
;;;    prmex
;;;  naus_aso
;;;  naus_chl
;;;  naus_chspace
;;;  naus_date
;;;  naus_delfst
;;;  naus_init_11
;;;  naus_lastent
;;;  naus_pspacep
;;;  naus_ptos
;;;  naus_rtosvd
;;;  naus_save
;;;  naus_setvar
;;;  naus_ssfrom
;;;  naus_strpad
;;;  naus_symbol_list
;;;  naus_tidy_11
;;; ==========================================================================


;; ---------------------------------------------------------------------------
;; Function: C:MACIFY
;; Purpose : AutoCAD command to explode all entities to the deepest possible
;;           level and place any PS entities into MS.
;; Global  : GR: selection set for new entities
;;           UXB: selection set for unexploded blocks
;; Local   : ss: selection set
;;           #: loop counter
;;           renamed: list of xrefs to be renamed
;;           renlist: list of lists of new and old renamed xref names
;;           blocks: list of blocks in drawing
;;           el1, el2: last entity (used as markers to identify new entities)
;;           xlist: list of XREFs bound in one iteration
;;           xlist2: list of XREFs bound in this command
;;           layers: list of layers in drawing
;;           new: new name of xref
;;           newlen: length of new name of xref
;;           fw: file pointer of report file to write
;; Uses    : (local_error) (get_xref) (add_xref) (move_ps) (report_file): local
;;           (mex_main) (naus_lastent) (naus_ssfrom)
;;             (naus_init_11) (naus_tidy_11): global
;; ---------------------------------------------------------------------------

(defun C:MACIFY (/ local_error get_xlay add_xref move_ps report_file
                   ss # renamed renlist blocks el1 el2 xlist xlist2 layers
                   new newlen fw)

;; ---------------------------------------------------------------------------
;; Function: LOCAL_ERROR
;; Purpose : Error handling function.
;; Params  : ST: string containing error description
;;           (naus_tidy_11): global
;; ---------------------------------------------------------------------------

  (defun local_error (ST)
    (if (not (or (= ST "Function cancelled") (= ST "console break")))
      (princ (strcat "\nError: " ST " "))
    )
    (naus_tidy_11)
  ) ; End local_error


;; ---------------------------------------------------------------------------
;; Function: GET_XLAY
;; Purpose : Gets a list of xrefs which have layers that would
;;           set.  Used to add newly bound XREFs to the entities to re-process.
;; Param   : NAME: xref/block name string
;;           SS: selection set to which to add insertions
;; Local   : ss2: selection set of insertions
;;           #: loop counter
;; ---------------------------------------------------------------------------

  (defun get_xlay (/ layers # lay ch xref xrefs)
    (setq
      layers (naus_symbol_list "LAYER")
      # 0
    )
    (while (setq lay (nth # layers))
      (if
        (and
          (setq ch (naus_chl "|" lay T))
          (> (strlen lay) 29)
          (setq xref (substr lay 1 (1- ch)))
          (not (member xref xrefs))
        )
        (setq xrefs (append (list xref) xrefs))
      )
      (setq # (1+ #))
    )
    xrefs
  ) ; End get_xlay


;; ---------------------------------------------------------------------------
;; Function: ADD_XREF
;; Purpose : Adds all insertions of a given block name to a given selection
;;           set.  Used to add newly bound XREFs to the entities to re-process.
;; Param   : NAME: xref/block name string
;;           SS: selection set to which to add insertions
;; Local   : ss2: selection set of insertions
;;           #: loop counter
;; ---------------------------------------------------------------------------

  (defun add_xref (NAME SS / ss2 #)
    (setq
      ss2 (ssget "X" (list '(0 . "INSERT") (cons 2 NAME)))
      # (sslength ss2)
    )
    (while (> # 0)
      (ssadd (ssname ss2 (setq # (1- #))) SS)
    )
  ) ; End add_xref


;; ---------------------------------------------------------------------------
;; Function: MOVE_PS
;; Purpose : Moves everything in PS to MS, except viewports.  Uses largest
;;           viewport as basis for move origin and scale.
;; Param   : SS: selection set of all objects in PS
;; Local   : #: loop counter
;;           ssvp: selection set of viewports
;;           en: entity name
;;           eg: (entget) list for en
;;           area: size of viewport in square PS drawing units
;;           biggest: list of area, plus eg
;;           eed: extended entity data
;;           l1040: list from eg, starting at first 1040 item
;; Uses    : (naus_pspacep) (naus_aso): global
;; ---------------------------------------------------------------------------

  (defun move_ps (SS / # ssvp en eg area biggest eed l1040)
    (if (not (naus_pspacep))
      (if (= (getvar "TILEMODE") 1)
        (setvar "TILEMODE" 0)
        (command ".PSPACE")
      )
    )
    (setq
      # 0
      ssvp (ssadd)
    )
    (while (setq en (ssname SS #))
      (if (= "VIEWPORT" (naus_aso 0 (entget en)))
        (progn
          (ssdel en SS)
          (ssadd en ssvp)
        )
      )
      (setq # (1+ #))
    )
    (if (and ssvp (> (sslength ssvp) 0))
      (progn
        (setq # 0)
        (while (setq en (ssname ssvp #))
          (setq
            eg (entget en '("ACAD"))
            area (* (naus_aso 40 eg) (naus_aso 41 eg))
          )
          (if
            (and
              (/= 1 (naus_aso 69 eg))
              (or (not biggest) (> area (car biggest)))
            )
            (setq biggest (list area eg))
          )
          (setq # (1+ #))
        )
        (setq
          eg (cadr biggest)
          eed (cdar (naus_aso -3 eg))
          l1040 (member (assoc 1040 eed) eed)
        )
      )
      (setq ssvp nil)
    )
    (if (and ssvp SS (> (sslength SS) 0))
      (progn
        (if (tblsearch "BLOCK" "MACIFY")
          (command ".BLOCK" "MACIFY" "Y" "NON" (naus_aso 10 eg) SS "")
          (command ".BLOCK" "MACIFY" "NON" (naus_aso 10 eg) SS "")
        )
        (command ".ERASE" ssvp "")
        (setvar "TILEMODE" 1)
        (command
          ".INSERT" "*MACIFY"
          (list (cdr (nth 2 l1040)) (cdr (nth 3 l1040)) 0.0)
          (/ (cdr (nth 1 l1040)) (naus_aso 41 eg)) 0
        )
      )
    )
  ) ; End move_ps


;; ---------------------------------------------------------------------------
;; Function: REPORT_FILE
;; Purpose : Writes a report to a file.  Closes the file.
;; Param   : FW: file pointer to an opened file set to write
;; Local   : lu: LUNITS setting
;;           units: string representing LUNITS setting
;;           lay: layer information from table
;; ---------------------------------------------------------------------------

  (defun report_file (FW / lu units lay)
    (setq
      lu (getvar "LUNITS")
      units
      (cond
        ((= lu 1) "Scientific")
        ((= lu 2) "Decimal")
        ((= lu 3) "Engineering")
        ((= lu 4) "Architectural")
        ((= lu 5) "Fractional")
      )
    )
    (write-line (strcat "File:       " (getvar "DWGNAME")) FW)
    (write-line (strcat "Date:       " (naus_date)) FW)
    (write-line (strcat "Platform:   " (getvar "PLATFORM")) FW)
    (write-line (strcat "Release:    " (getvar "ACADVER")) FW)
    (write-line (strcat "Login name: " (getvar "LOGINNAME")) FW)
    (write-line (strcat "Serial no:  " (getvar "_PKSER")) FW)
    (write-line (strcat "Units:      " units) FW)
    (write-line (strcat "Ltscale:    " (rtos (getvar "LTSCALE") 2 2)) FW)
    (write-line (strcat "Dimscale:   " (rtos (getvar "DIMSCALE") 2 2)) FW)
    (write-line
      (strcat
        "Extents:    " (rtos (car (getvar "EXTMIN")) 2 2)
        "," (rtos (cadr (getvar "EXTMIN")) 2 2)
        " to " (rtos (car (getvar "EXTMAX")) 2 2)
        "," (rtos (cadr (getvar "EXTMAX")) 2 2)
      )
      FW
    )
    (write-line "Layers:" FW)
    (setq lay (tblnext "LAYER" T))
    (while lay
      (write-line
        (strcat
          " " (naus_strpad (naus_aso 2 lay) " " 31 T)
          " " (if (< (naus_aso 62 lay) 0) "Off" "   ")
          " " (if (= 1 (logand 1 (naus_aso 70 lay))) "Frozen" "      ")
          " " (if (= 4 (logand 4 (naus_aso 70 lay))) "Locked" "")
        )
        FW
      )
      (setq lay (tblnext "LAYER"))
    )
    (close FW)
  ) ; End report_file

;; Start C:MACIFY ------------------------------------------------------------

  (naus_init_11 nil nil local_error)
  (if
    (and
      (setq ss (ssget "X" '((67 . 1))))
      (> (sslength ss) 0)
      (<= (sslength (ssget "X" '((0 . "VIEWPORT")))) 1)
    )
    (progn
      (prompt
        "\nIn paper space there are entities but no viewports.  Creating one. "
      )
      (naus_chspace 1)
      (command ".MVIEW" "NON" (getvar "VSMIN") "NON" (getvar "VSMAX"))
    )
  )
  (setq
    ss
    (ssget "X"
      '((-4 . "<OR")
         (0 . "INSERT") (0 . "DIMENSION") (0 . "POLYLINE") (0 . "MTEXT")
        (-4 . "OR>"))
    )
    el1 (naus_lastent)
    UXB (ssadd)
    renamed (get_xlay)
    blocks (naus_symbol_list "BLOCK")
    # 0
  )
  (foreach xref renamed
    (while (member (setq new (strcat "X_" (itoa #))) blocks)
      (setq # (1+ #))
    )
    (setq renlist (append (list (list new xref)) renlist))
    (command ".RENAME" "Block" xref new)
  )
  (prompt "\nMACIFY explodes things in several passes.  Pass:\n")
  (setq # 1)
  (while ss
    (prompt (strcat "\r " (itoa #) " "))
    (setq
      el2 (naus_lastent)
      xlist (mex_main ss "All" nil)
      xlist2 (append xlist xlist2)
      ss (naus_ssfrom el2)
      # (1+ #)
    )
    (foreach xref xlist ; Add any bound XREFs to selection set to be processed
      (add_xref xref ss)
    )
  )
  (if (setq GR (naus_ssfrom el1))
    (prompt "\nNew entities are in selection set GR. ")
    (prompt "\nNothing exploded. ")
  )
  (if (> (sslength UXB) 0)
    (prompt "\nUnexploded blocks are in selection set UXB. ")
  )
  (if xlist2
    (progn
      (prompt "\nThe following XREFs have been bound:\n")
      (foreach xref xlist2
        (prompt
          (strcat " "
            (if (assoc xref renlist) (cadr (assoc xref renlist)) xref)
          )
        )
      )
      (prompt "\n")
    )
  )
  (setq layers (naus_symbol_list "LAYER"))
  (foreach xref renlist
    (setq
      new (car xref)
      newlen (strlen new)
    )
    (foreach layer layers
      (if
        (and
          (= (strcat new "$") (substr layer 1 (1+ newlen)))
          (= "$" (substr layer (+ 3 newlen ) 1))
        )
        (command
          ".RENAME" "Layer" layer
          (strcat
            (cadr xref)
            (substr layer (+ 2 newlen) 1)
            (substr layer (+ 4 newlen))
          )
        )
      )
    )
  )
  (if (and (setq ss (ssget "X" '((67 . 1)))) (> (sslength ss) 0))
    (progn
      (prompt "\nMoving paper space entities to model space... ")
      (move_ps ss)
    )
  )
  (command ".ZOOM" "E")
  (if (setq fw (open (strcat (getvar "DWGNAME") ".DX!") "w"))
    (setq fw (report_file fw))
  )
  (prompt "\nFinished!")
  (naus_tidy_11)
) ; End C:MACIFY


;; ---------------------------------------------------------------------------
;; Function: MEX_MAIN
;; Purpose : Main section of MEX: provides API for AutoCAD commands.
;; Params  : SS: selection set (may contain unexplodable entities)
;;           ETYP: entity type string ("Block" "Dimension" "Polyline" "Xref")
;;           SHOW: if T, messages are displayed.  If nil, messages supressed.
;; Global  : UXB: selection set for unexploded blocks
;; Local   : #: loop counter
;;           en: entity name
;;           eg: (entget) list for entity
;;           et: entity type from entity
;;           xlist: list of XREFs bound in this command
;; Returns : xlist (nil if no XREFs bound)
;; Uses    : (exblk) (exdim) (expol) (exmtext) (prmex): local
;;           (naus_aso) (naus_chspace): global
;; ---------------------------------------------------------------------------

(defun mex_main (SS ETYP SHOW / expol exdim exblk exmtext fixnew prmex
                                # en eg et xlist)


;; ---------------------------------------------------------------------------
;; Function: EXBLK
;; Purpose : Explode a block
;; Param   : EGBLK: (entget) list of block
;; Local   : el: last entity in database before explosion
;;           en: entity name of block insertion
;;           bn: block name string
;;           blklst: list of information about block
;;           p1: block insertion point
;;           p2: second point on mirror line
;;           sa: snap angle before adjustment for array
;;           al: attribute list containing tags and values
;; Uses    : (atfix) (atlist): local
;;           (fixnew) (prmex)
;;           (naus_aso) (naus_lastent) (naus_ssfrom): global
;; ---------------------------------------------------------------------------
  
  (defun exblk (EGBLK / atfix atlist
                        el en bn blklst p1 p2 sa al)


;; ---------------------------------------------------------------------------
;; Function: ATFIX
;; Purpose : Fix up attribute definitions from a given entity onwards using a
;;           list of attribute tags and values.  Deletes ATTDEFs and creates
;;           TEXT entities based on attribute list.
;; Params  : EL: entity after which to start processing
;;           AL: attribute list containing (entget) lists of old ATTRIBs
;; Local   : en: entity name
;;           atdisp: current setting of "ATTMODE" system variable
;;           a74: (assoc) list from eg
;; Foreach : eg: (entget) list from AL
;; Uses    : (naus_aso) (naus_delfst): global
;; ---------------------------------------------------------------------------
  
    (defun atfix (EL AL / en atdisp a74)
      (setq en EL)
      (while (setq en (entnext en))
        (if (= "ATTDEF" (naus_aso 0 (entget en))) ; Found ATTDEF
          (entdel en)                        ; Delete it
        )
      )
      (if (/= 0 (setq atdisp (getvar "ATTMODE")))
        (foreach eg AL
          (if (or (= 2 atdisp) (/= 1 (logand 1 (naus_aso 70 eg))))
            (progn
              (setq
                eg (subst (cons 0 "TEXT") (assoc 0 eg) eg)
                eg (naus_delfst (assoc 2 eg) eg)
                eg (naus_delfst (assoc 70 eg) eg)
                eg (naus_delfst (assoc 73 eg) eg)
              )
              (if (setq a74 (assoc 74 eg))
                (setq eg (subst (cons 73 (cdr a74)) a74 eg))
              ) 
              (entmake eg)
            )
          )
        )
      )
    ) ; End atfix
    

;; ---------------------------------------------------------------------------
;; Function: ATLIST
;; Purpose : Returns a list of attributes contained in a block, excluding
;;           those with values which are empty or contain only spaces.
;; Param   : EGBLK: (entget) list for block
;; Local   : en: entity name of block subentity
;;           eg: (entget) list for en
;;           retlst: list to return with each sublist as the (entget) list
;; Uses    : (naus_aso): global
;; ---------------------------------------------------------------------------
    
    (defun atlist (EGBLK / en eg retlst)
      (setq en (entnext (naus_aso -1 EGBLK)))
      (while (/= "SEQEND" (naus_aso 0 (setq eg (entget en))))
        (if (and (/= (naus_aso 1 eg) "") (not (wcmatch (naus_aso 1 eg) " ")))
          (setq retlst (append (list eg) retlst))
        )
        (setq en (entnext en))
      )
      retlst
    ) ; End atlist
    

;; Start exblk -------------------

    (prmex
      (strcat "\nBlock " (naus_aso 2 EGBLK) " on layer " (naus_aso 8 EGBLK))
    )
    (setq
      el (naus_lastent)
      en (naus_aso -1 EGBLK)
      bn (naus_aso 2 EGBLK)
      blklst
      (list
        (naus_aso 44 EGBLK)         ; 0 Column distance
        (naus_aso 45 EGBLK)         ; 1 Row distance
        (naus_aso 50 EGBLK)         ; 2 Rotation angle
        (max 1 (naus_aso 70 EGBLK)) ; 3 Number of columns
        (max 1 (naus_aso 71 EGBLK)) ; 4 Number of rows
        (naus_aso 66 EGBLK)         ; 5 Attributes follow flag
      )
    )
    (cond
      ((= 4 (logand 4 (naus_aso 70 (tblsearch "BLOCK" bn)))) ; XREF block
        (if (= "Block" ETYP)
          (prmex " is an XREF, ignored")
          (progn
            (prmex " is an XREF.  NOTE: binding all XREFs of this name")
            (command ".XREF" "B" bn)
            (setq xlist (cons bn xlist))
          )
        )
      )
      ((member bn xlist)                                ; Bound XREF
        (prmex " is an XREF - already bound")
      )
      ((= "Xref" ETYP) ; This stops the rest of the (cond) being processed
        (prmex " is not an XREF, ignored")
      )
      ((> (max (nth 3 blklst) (nth 4 blklst)) 1)        ; MINSERT block
        (prmex ", MINSERTed")
        (setq
          sa (getvar "SNAPANG")
          EGBLK (subst (cons 44 0.0) (assoc 44 EGBLK) EGBLK)
          EGBLK (subst (cons 45 0.0) (assoc 45 EGBLK) EGBLK)
          EGBLK (subst (cons 70 0) (assoc 70 EGBLK) EGBLK)
          EGBLK (subst (cons 71 0) (assoc 71 EGBLK) EGBLK)
        )
        (entmod EGBLK)
        (setvar "SNAPANG" 0)
        (command
          ".UCS" "E" en
          ".ARRAY" en "" "R" (nth 4 blklst) (nth 3 blklst)
        )
        (if (> (nth 4 blklst) 1)
          (command (nth 1 blklst))
        )
        (if (> (nth 3 blklst) 1)
          (command (nth 0 blklst))
        )
        (setvar "SNAPANG" sa)
        (command ".UCS" "P")
      )
      ((= (naus_aso 41 EGBLK) (naus_aso 42 EGBLK) (naus_aso 43 EGBLK)) ; Block
        (if (= (nth 5 blklst) 1)
          (progn                                        ; With attributes
            (prmex " with attributes")
            (setq al (atlist EGBLK))                    ; so get tags, values
          )
        )
        (command ".EXPLODE" en)
        (if (= (nth 5 blklst) 1) (atfix el al))         ; Fix up attributes
      )
      ((= (- (naus_aso 41 EGBLK)) (naus_aso 42 EGBLK))  ; Mirrored block
        (prmex ", MIRRORed")
        (if (= (nth 5 blklst) 1)
          (progn                                        ; With attributes
            (prmex " with attributes")
            (setq al (atlist EGBLK))                    ; so get tags, values
          )
        )
        (command
          ".MIRROR" en "" (setq p1 (naus_aso 10 EGBLK))
          (setq p2 (polar p1 (naus_aso 50 EGBLK) 1)) "Y"
          ".EXPLODE" en
          "MIRROR" (naus_ssfrom el) "" p1 p2 "Y"
        )
        (if (= (nth 5 blklst) 1) (atfix el al))         ; Fix up attributes
      )
      (T                                                ; Unexplodable block
        (prmex " is unexplodable")
        (ssadd en UXB)                                  ; so add it to UXB
      )
    )
    (fixnew el EGBLK)                                   ; fix new entities
  ) ; End exblk
  

;; ---------------------------------------------------------------------------
;; Function: EXDIM
;; Purpose : Explode a dimension
;; Param   : EG: (entget) list of dimension
;; Local   : el: last entity in database before explosion
;; Uses    : (naus_aso) (naus_lastent): global
;;           (fixnew)
;; ---------------------------------------------------------------------------
  
  (defun exdim (EG / el)
    (setq el (naus_lastent))
    (prmex (strcat "\nDimension on layer " (naus_aso 8 EG)))
    (command ".EXPLODE" (naus_aso -1 EG))
    (fixnew el EG)          ; fix new layer 0 entities
  )


;; ---------------------------------------------------------------------------
;; Function: EXMTEXT
;; Purpose : Explode mtext
;; Param   : EG: (entget) list of mtext
;; Uses    : (naus_aso): global
;; ---------------------------------------------------------------------------

  (defun exmtext (EG)
    (prmex (strcat "\nMtext on layer " (naus_aso 8 EG)))
    (command ".EXPLODE" (naus_aso -1 EG))
  ) ; End exmtext


;; ---------------------------------------------------------------------------
;; Function: EXPOL
;; Purpose : Explode a polyline
;; Param   : EG: (entget) list of polyline
;; Uses    : (naus_aso): global
;; ---------------------------------------------------------------------------

  (defun expol (EG)
    (prmex (strcat "\nPolyline on layer " (naus_aso 8 EG)))
    (command ".EXPLODE" (naus_aso -1 EG))
  ) ; End expol


;; ---------------------------------------------------------------------------
;; Function: FIXNEW
;; Purpose : Changes layer, colour, linetype of new layer 0 BYBLOCK entities.
;;           Changes the layer, colour and linetype to that of the parent
;;           block.  Deletes any POINT entities it finds on layer DEFPOINTS.
;; Params  : EL: entity after which to start processing
;;           EGBLK: (entget) list of parent block
;; Local   : ss: selection set of new entities (after EL)
;;           #: loop counter
;;           en: entity name of a new entity
;;           oldeg, eg: original (entget) list from en, modified version
;;           la, col, lt: layer, colour and linetype of parent block
;; Uses    : (naus_ssfrom) (naus_aso): global
;; ---------------------------------------------------------------------------

  (defun fixnew (EL EGBLK / ss # en oldeg eg la col lt)
    (if (setq ss (naus_ssfrom EL))
      (progn
        (setq ; Get properties of original entity
          la (naus_aso 8 EGBLK)
          col (naus_aso 62 EGBLK)
          lt (naus_aso 6 EGBLK)
          # 0.0
        )
        (if (not col) (setq col 256))
        (if (not lt) (setq lt "BYLAYER"))
        (while (setq en (ssname ss #))
          (setq
            oldeg (entget en)
            eg oldeg
          )
          (if (and (= "DEFPOINTS" (naus_aso 8 eg)) (= "POINT" (naus_aso 0 eg)))
            (entdel en)     ; Dimension point?  Yes, delete it
            (progn          ; Check for layer 0, BYBLOCK properties
              (if (= "0" (naus_aso 8 eg))        ; Layer
                (setq eg (subst (cons 8 la) (assoc 8 eg) eg))
              )
              (if (= 0 (naus_aso 62 eg))         ; Colour
                (setq eg (subst (cons 62 col) (assoc 62 eg) eg))
              )
              (if (= "BYBLOCK" (naus_aso 6 eg))  ; Linetype
                (setq eg (subst (cons 6 lt) (assoc 6 eg) eg))
              )
              (if (/= eg oldeg)
                (entmod eg)
              )
            )
          )
          (setq # (1+ #))
        )
      )
    )
  ) ; End fixnew


;; Start mex_main ------------------------------------------------------------

  (setq # 0.0) ; use real as index to allow large selection sets
  (if (or (not UXB) (/= 'PICKSET (type UXB)))
    (setq UXB (ssadd)) ; Initialise UXB if not already done
  )
  (if SHOW ; Set up (prmex) for showing messages (or not if SHOW = nil)
    (setq prmex princ)
    (defun prmex (DUMMY) nil)
  )
  (setvar "BLIPMODE" 0)
  (setvar "HIGHLIGHT" 0)
  (setvar "OSMODE" 0)
  (while (setq en (ssname SS #)) ; Get each entity
    (setq
      eg (entget en)
      et (naus_aso 0 eg)
    )
    (naus_chspace (naus_aso 67 eg))
    (cond
      ((and
         (= et "INSERT")
         (or (not ETYP) (= ETYP "All") (= ETYP "Xref") (= ETYP "Block"))
       )
        (exblk eg)
      )
      ((and
         (= et "DIMENSION")
         (or (not ETYP) (= ETYP "All") (= ETYP "Dimension"))
       )
        (exdim eg)
      )
      ((and
         (= et "POLYLINE")
         (or (not ETYP) (= ETYP "All") (= ETYP "Polyline"))
       )
        (expol eg)
      )
      ((and
         (= et "MTEXT")
         (or (not ETYP) (= ETYP "All") (= ETYP "Mtext"))
       )
        (exmtext eg)
      )
    )
    (setq # (1+ #))
  )
  xlist
) ; End mex_main


;; ---------------------------------------------------------------------------
;; Function: NAUS_ASO
;; Purpose : Short form of (cdr (assoc)).
;; Params  : ID: identifying number in association list
;;           AL: association list
;; Returns : Data section of association list item.
;; ---------------------------------------------------------------------------

(defun naus_aso (ID AL)
  (cdr (assoc ID AL))
) ; End naus_aso


;; ---------------------------------------------------------------------------
;; Function: NAUS_CHL
;; Purpose : Finds the last occurance of a certain character in a string 
;;           or the last character in a string which is different to the
;;           specified character.
;; Call    : (naus_chl CH <Char> ST <String> MATCH <Boolean>)
;; Examples: (naus_chl "x" "ABxDEx" T) returns 6
;;           (naus_chl "x" "ABCDEF" T) returns nil
;;           (naus_chl "x" "" T) returns nil
;;           (naus_chl "x" "xBCxxx" nil) returns 3
;;           (naus_chl "x" "ABCDEF" nil) returns 6
;;           (naus_chl "x" "xxxxxx" nil) returns nil
;;           (naus_chl "x" "" nil) returns nil
;; Local   : #: loop counter
;; Returns : (MATCH = T)   Position of last character if found, else nil
;;           (MATCH = nil) Position of last non-ch character if found, else nil
;; ---------------------------------------------------------------------------

(defun naus_chl (CH ST MATCH / #)
  (setq
    # (strlen ST)
  )
  (if MATCH
    (while (and (> # 0) (/= CH (substr ST # 1)))
      (setq # (1- #))
    )
    (while (and (> # 0) (= CH (substr ST # 1)))
      (setq # (1- #))
    )
  )
  (if (> # 0) #)
) ; End naus_chl


;; ---------------------------------------------------------------------------
;; Function: NAUS_CHSPACE
;; Purpose : Changes to the same space as the entity whose 67 code is passed in
;; Param   : CODE67: 67 code of entity. 1 indicates PS, nil or 0 indicates MS.
;; Uses    : (naus_pspacep): global
;; ---------------------------------------------------------------------------

(defun naus_chspace (CODE67)
  (if (naus_pspacep)
    (if (/= CODE67 1)
      (if (> (sslength (ssget "X" '((0 . "VIEWPORT")))) 1)
        (command ".MSPACE")
        (setvar "TILEMODE" 1)
      )
    )
    (if (= CODE67 1)
      (if (= (getvar "TILEMODE") 1)
        (setvar "TILEMODE" 0)
        (command ".PSPACE")
      )
    )
  )
) ; End naus_chspace


;; ---------------------------------------------------------------------------
;; Function: NAUS_DATE
;; Purpose : Returns the current date as a string in DD Mon YYYY format
;; ---------------------------------------------------------------------------

(defun naus_date (/ cdate year month day)
  (setq
    cdate (rtos (getvar "CDATE") 2 0)
    year (substr cdate 1 4)
    month
    (nth
      (1- (atoi (substr cdate 5 2)))
     '("Jan" "Feb" "Mar" "Apr" "May" "Jun" "Jul" "Aug" "Sep" "Oct" "Nov" "Dec")
    )
    day (substr cdate 7 2)
  )
  (strcat day " " month " " year)
)

;; ---------------------------------------------------------------------------
;; Function: NAUS_DELFST
;; Purpose : Returns a list with the first occurence of a given item deleted
;; Params  : ITEM:   item to delete
;;           LST:    list to delete from
;; Local   : it1:    first item in list
;;           notfnd: boolean: T while item not found
;;           tlst:   temporary list
;; Examples: (naus_delfst 5 '(1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6))
;;           returns => (1 2 3 4 6 7 8 9 0 1 2 3 4 5 6)
;;           (naus_delfst 5 '(1 2 3 4))  => (1 2 3 4)
;;           (naus_delfst 5 nil)         => nil
;; ---------------------------------------------------------------------------

(defun naus_delfst (ITEM LST / it1 notfnd tlst)
  (setq notfnd T)
  (while (and LST notfnd)
    (setq
      it1 (car LST)
      LST (cdr LST)
    )
    (if (/= ITEM it1)
      (setq tlst (cons it1 tlst))
      (setq
        notfnd nil
        tlst (append (reverse LST) tlst)
      )
    )
  )
  (reverse tlst)
) ; End naus_delfst


;; ---------------------------------------------------------------------------
;; Function: NAUS_INIT_11
;; Purpose : Sets up at the start of a command.  Assumes (naus_tidy_11) will be
;;           used.
;; Params  : SEL: list of entity types to restrict selection (not used)
;;           VARS: list of system variables to save
;;           ERR: new error handling function
;; Global  : NAUS-OLD-VARS: saved list of system variables and values
;;           NAUS-OLD-ERROR: saved *error* function
;; Uses    : (naus_setvar)
;; ---------------------------------------------------------------------------

(defun naus_init_11 (SEL VARS ERR / ss)
  (setq
    NAUS-OLD-VARS (if VARS (naus_save VARS))
    NAUS-OLD-ERROR *error*
;; Comment out the next line to disable error handler for debugging
    *error* ERR
  )
  (setvar "CMDECHO" 0)
  (command ".UNDO" "Group")
  ss
) ; End naus_init_11


;; ---------------------------------------------------------------------------
;; Function: NAUS_LASTENT
;; Purpose : Gets the last entity or sub-entity.  More useful than (entlast)
;;           which just gets the last entity.
;; ---------------------------------------------------------------------------

(defun naus_lastent (/ en entemp)
  (setq
    entemp (entlast)
    en (entnext entemp)
  )
  (while en
    (setq
      entemp en
      en (entnext entemp)
    )
  )
  entemp
) ; End naus_lastent


;; ---------------------------------------------------------------------------
;; Function: NAUS_PSPACEP
;; Purpose : Paper space predicate: returns T if currently in paper space.
;; ---------------------------------------------------------------------------

(defun naus_pspacep ()
  (and (= (getvar "TILEMODE") 0) (= (getvar "CVPORT") 1))
) ; End naus_pspacep


;; ---------------------------------------------------------------------------
;; Function: NAUS_PTOS
;; Purpose : Converts a point to a string "X,Y" or "X,Y,Z"
;; Call    : (naus_ptos PT <Point>)
;; Example : (naus_ptos '(1.0 2.0 3.0))
;;           (prompt (strcat "\nPoint selected was " (naus_ptos p1)))
;; Params  : PT: point list
;; Returns : String (e.g. "1.0,2.0,3.0")
;; Method  : rtos on (car PT) and (cadr PT) and possibly (caddr PT)
;;           Uses (naus_rtosvd) to ensure enough decimal places are used.
;; ---------------------------------------------------------------------------

(defun naus_ptos (PT)
  (strcat
    (naus_rtosvd (car PT)) "," (naus_rtosvd (cadr PT))
    (if (caddr PT) (strcat "," (naus_rtosvd (caddr PT))) "")
  )
) ; End naus_ptos


;; ---------------------------------------------------------------------------
;; Function: NAUS_RTOSVD
;; Purpose : Real to string with variable number of decimal places.  Returns
;;           a string containing the number using the number of decimal places
;;           needed to naus_display the number.  Uses a minimum of the current
;;           LUPREC setting and a maximum of 8 decimal places.  Always uses
;;           decimals.
;; Params  : VAL: real number to convert
;; Local   : luprec: current value of sysvar LUPREC
;; ---------------------------------------------------------------------------

(defun naus_rtosvd (VAL / luprec)
  (setq luprec (getvar "LUPREC"))
  (while (and (/= VAL (distof (rtos VAL 2 luprec))) (< luprec 8))
    (setq luprec (1+ luprec))
  )
  (rtos VAL 2 luprec)
) ; End naus_rtosvd


;; ---------------------------------------------------------------------------
;; Function: NAUS_SAVE
;; Purpose : Saves the values of specified system variables.
;; Params  : VARS: list of strings representing system variables
;; Local   : varlst: temporary list of variables
;; Returns : list of lists, each containing a variable name and value
;; ---------------------------------------------------------------------------

(defun naus_save (VARS / varlst)
  (foreach one VARS
    (setq varlst (append (list (list one (getvar one))) varlst))
  )
  varlst
) ; End naus_save


;; ---------------------------------------------------------------------------
;; Function: NAUS_SETVAR
;; Purpose : Sets the specified system variables to the specified values.
;; Params  : VARS: list of lists, each containing a variable name and value
;; ---------------------------------------------------------------------------

(defun naus_setvar (VARS)
  (foreach one VARS
    (setvar (car one) (cadr one))
  )
) ; End naus_setvar


;; ---------------------------------------------------------------------------
;; Function: NAUS_SSFROM
;; Purpose : Selects all objects drawn after a given entity.
;; Author  : S J Johnson
;; Date    : 12 March 1990
;; Call    : (naus_ssfrom EN <Entity name>)
;; Example : (setq L (entlast))
;;           (command "LINE" .... etc)
;;           (setq SS (naus_ssfrom L)) - selects all entities drawn after l
;; Params  : EN: name of entity after which to start selecting
;; Local   : ss: selection set
;; Returns : selection set
;; Notes   : If EN is nil, (e.g. (entlast) returns nil because nothing has been
;;           drawn yet), returns selection set containing the whole drawing.
;;           This eliminates the need to ensure something is drawn before using
;;           (entlast).
;; ---------------------------------------------------------------------------

(defun naus_ssfrom (EN / ss)
  (if EN
    (progn
      (setq ss (ssadd))
      (while (setq EN (entnext EN))            
        (ssadd EN ss)
      )
    )
    (ssget "X")
  )
) ; End naus_ssfrom


;; ---------------------------------------------------------------------------
;; Function: NAUS_STRPAD
;; Purpose : Returns a string, padded with a character to a fixed length,
;;           or chopped off to that length.
;; Params  : STRING: string to pad/chop
;;           CHAR: character to pad with
;;           LEN: length of string
;;           END: if T, modify end of string.  If nil, modify start.
;; ---------------------------------------------------------------------------

(defun naus_strpad (STRING CHAR LEN END / sl retstr)
  (setq
    sl (strlen STRING)
    retstr STRING
  )
  (if (>= sl LEN)
    (if END
      (substr STRING 1 LEN)
      (substr STRING (- sl LEN -1))
    )
    (repeat (- LEN sl)
      (if END
        (setq retstr (strcat retstr CHAR))
        (setq retstr (strcat CHAR retstr))
      )
    )
  )
) ; End naus_strpad

;; ---------------------------------------------------------------------------
;; Function: NAUS_SYMBOL_LIST
;; Purpose : Returns a list of all items in a symbol table.
;; ---------------------------------------------------------------------------

(defun naus_symbol_list (TABLE / symbol symbol-list)
  (setq symbol (tblnext TABLE T))
  (while symbol
    (setq
      symbol-list (cons (naus_aso 2 symbol) symbol-list)
      symbol (tblnext TABLE)
    )
  )
  symbol-list
) ; End naus_symbol_list


;; ---------------------------------------------------------------------------
;; Function: NAUS_TIDY_11
;; Purpose : Tidies up after a command.  Assumes (naus_init) has been used.
;; Global  : NAUS-OLD-VARS: saved list of system variables and values
;;           NAUS-OLD-ERROR: saved *error* function
;; Uses    : (naus_setvar)
;; ---------------------------------------------------------------------------

(defun naus_tidy_11 ()
  (naus_setvar NAUS-OLD-VARS)
  (setq
    *error* NAUS-OLD-ERROR
    NAUS-OLD-ERROR nil
    NAUS-OLD-VARS nil
  )
  (command ".UNDO" "End")
  (princ)
) ; End naus_tidy_11


(princ)

;;; ==========================================================================
;;; End file MACIFY.LSP
;;; ==========================================================================